@@ -13,12 +13,15 @@ class SyncSession:
     def __init__(self, session_store):
         self.session_store = session_store
         self.started = self.session_store.started
-        self.get = self.session_store.get
         self.next = self.session_store.next_item
         self.item = None
         self.filters = self.session_store.filters
 
-    
+
+    def get(self, k):
+        return self.session_store.get(str(k))
+   
+
     def start(self, offset=0, target=-1):
         self.session_store.start(offset=offset, target=target)
         self.item = self.session_store.next_item()
