@@ -13,6 +13,7 @@ from chainsyncer.error import (
         InterruptError,
         IncompleteFilterError,
         SyncDone,
+        FilterInitializationError,
         )
 
 logg = logging.getLogger(__name__)
@@ -42,7 +43,6 @@ class SyncItem:
         self.filter_state = filter_state
         self.state_key = str(offset)
 
-        logg.debug('get key {}'.format(self.state_key))
         v = self.sync_state.get(self.state_key)
 
         (self.cursor, self.tx_cursor, self.target) = sync_state_deserialize(v)
@@ -66,16 +66,6 @@ class SyncItem:
             raise FilterDone(self.state_key)
 
 
-    def resume(self):
-        filter_state = self.filter_state.state(self.state_key)
-        if filter_state > 0x0f:
-            filter_state_part = self.filter_state.mask(filter_state, 0x0f)
-            if len(self.filter_state.elements(filter_state)) == 1:
-                logg.info('resume execution on state {} ({})'.format(self.filter_state.name(filter_state_part), filter_state_part))
-                lock_state = self.filter_state.from_name('LOCK')
-                self.filter_state.set(lock_state)
-
-
     def reset(self, check_incomplete=True):
         if check_incomplete:
             if self.filter_state.state(self.state_key) & self.filter_state.from_name('LOCK') > 0:
@@ -110,10 +100,6 @@ class SyncItem:
         self.sync_state.replace(self.state_key, b)
 
 
-    def __find_advance(self):
-        v = self.filter_state.state(self.state_key)
-
-
     def advance(self, ignore_lock=False):
         if self.skip_filter:
             raise FilterDone()
@@ -221,9 +207,9 @@ class SyncStore:
             self.state.put(block_number_str, contents=state_bytes)
             self.filter_state.put(block_number_str)
             o = SyncItem(offset, target, self.state, self.filter_state, ignore_lock=ignore_lock)
-            o.resume()
-            self.items[offset] = o
-            self.item_keys.append(offset)
+            k = str(offset)
+            self.items[k] = o
+            self.item_keys.append(k)
         elif offset > 0:
             logg.warning('block number argument {} for start ignored for already initiated sync {}'.format(offset, self.session_id))
         self.started = True
@@ -271,12 +257,14 @@ class SyncStore:
             if i < lim:
                 item_target = thresholds[i+1] 
             o = SyncItem(block_number, item_target, self.state, self.filter_state, started=True, ignore_lock=ignore_lock)
-            o.resume()
-            self.items[block_number] = o
-            self.item_keys.append(block_number)
+            k = str(block_number)
+            self.items[k] = o
+            self.item_keys.append(k)
             logg.info('added existing {}'.format(o))
 
-        self.get_target()
+        v = self.get_target()
+        if v != None:
+            target = v
         
         if len(thresholds) == 0:
             if self.target != None:
@@ -315,8 +303,73 @@ class SyncStore:
         raise NotImplementedError()
 
 
-    def peek_next_filter(self):
-        pass
+    def __get_locked_item(self):
+        locked_item = self.filter_state.list(self.filter_state.state_store.LOCK)
+        
+        if len(locked_item) == 0:
+            logg.error('Sync filter in store {} is not locked\n'.format(self))
+            return None
+        elif len(locked_item) > 1:
+            raise FilterInitializationError('More than one locked filter item encountered in store {}. That should never happen, so I do not know what to do next.\n'.format(self))
+        return locked_item[0]
+
+
+    def __get_filter_index(self, k):
+        i = -1
+        fltrs = self.load_filter_list()
+        for fltr in fltrs:
+            i += 1
+            if k == fltr.upper():
+                logg.debug('lock filter match at filter list index {}'.format(i))
+        return (i, fltrs,)
+
+
+    def unlock_filter(self, revert=False):
+        locked_item_key = self.__get_locked_item()
+        if locked_item_key == None:
+            return False
+        locked_item = self.get(locked_item_key)
+        state = self.filter_state.state(locked_item_key)
+        locked_state = state - self.filter_state.state_store.LOCK
+        locked_state_name = self.filter_state.name(locked_state)
+
+        logg.debug('found locked item {} in state {}'.format(locked_item, locked_state))
+
+        (i, fltrs) = self.__get_filter_index(locked_state_name)
+
+        if i == -1:
+            raise FilterInitializationError('locked state {} ({}) found for item {}, but matching filter has not been registered'.format(locked_state_name, locked_state, locked_item))
+
+        direction = None
+        if revert:
+            self.__unlock_previous(locked_item, fltrs, i)
+            new_state = self.filter_state.state(locked_item_key)
+            direction = 'previous'
+        else:
+            self.__unlock_next(locked_item, fltrs, i)
+            new_state = self.filter_state.state(locked_item_key)
+            direction = 'next'
+
+        logg.info('chainstate unlock to {} {} ({}) -> {} ({})'.format(direction, self.filter_state.name(state), state, self.filter_state.name(new_state), new_state))
+
+        return True
+
+
+    def __unlock_next(self, item, lst, index):
+        if index == len(lst) - 1:
+            item.reset(check_incomplete=False)
+        else:
+            item.release()
+
+
+    def __unlock_previous(self, item, lst, index):
+        if index == 0:
+            item.reset(check_incomplete=False)
+        else:
+            new_state_str = lst[index - 1]
+            new_state = self.filter_state.state_store.from_name(new_state_str)
+            self.filter_state.state_store.move(item.state_key, new_state)
+
 
     def peek_current_filter(self):
         pass
